@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2013-2016 Kevin Steves <kevin.steves@pobox.com>
+# Copyright (c) 2013-2017 Kevin Steves <kevin.steves@pobox.com>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -77,6 +77,7 @@ _wildfire_responses = {
 BENIGN = 0
 MALWARE = 1
 GRAYWARE = 2
+PHISHING = 4
 PENDING = -100
 ERROR = -101
 UNKNOWN = -102
@@ -86,6 +87,7 @@ VERDICTS = {
     BENIGN: ('benign', None),
     MALWARE: ('malware', None),
     GRAYWARE: ('grayware', None),
+    PHISHING: ('phishing', None),
     PENDING: ('pending', 'sample exists and verdict not known'),
     ERROR: ('error', 'sample is in error state'),
     UNKNOWN: ('unknown', 'sample does not exist'),
@@ -136,6 +138,7 @@ class PanWFapi:
 
         self._log(DEBUG3, 'Python version: %s', sys.version)
         self._log(DEBUG3, 'xml.etree.ElementTree version: %s', etree.VERSION)
+        self._log(DEBUG3, 'ssl: %s', ssl.OPENSSL_VERSION)
         self._log(DEBUG3, 'pan-python version: %s', __version__)
 
         if self.timeout is not None:
@@ -150,10 +153,16 @@ class PanWFapi:
             try:
                 ssl.SSLContext(ssl.PROTOCOL_SSLv23)
             except AttributeError:
-                raise PanXapiError('SSL module has no SSLContext()')
+                raise PanWFapiError('SSL module has no SSLContext()')
         elif _have_certifi:
             self.ssl_context = self._certifi_ssl_context()
 
+        # handle Python versions with no ssl.CertificateError
+        if hasattr(ssl, 'CertificateError'):
+            self._certificateerror = ssl.CertificateError
+        else:
+            self._certificateerror = NotImplementedError  # XXX Can't happen
+
         init_panrc = {}  # .panrc args from constructor
         if hostname is not None:
             init_panrc['hostname'] = hostname
@@ -222,6 +231,9 @@ class PanWFapi:
         elif content_type == 'text/html':
             return self.__set_html_response(message_body)
 
+        elif content_type == 'text/plain':
+            return self.__set_txt_response(message_body)
+
         else:
             msg = 'no handler for content-type: %s' % content_type
             self._msg = msg
@@ -281,6 +293,18 @@ class PanWFapi:
 
         return True
 
+    def __set_txt_response(self, message_body):
+        self._log(DEBUG2, '__set_txt_response: %s', repr(message_body))
+        self.response_type = 'txt'
+
+        _message_body = message_body.decode()
+        if len(_message_body) == 0:
+            return True
+
+        self.response_body = _message_body
+
+        return True
+
     # XXX store tostring() results?
     # XXX rework this
     def xml_root(self):
@@ -336,6 +360,9 @@ class PanWFapi:
 
         try:
             response = self._urlopen(**kwargs)
+        except self._certificateerror as e:
+            self._msg = 'ssl.CertificateError: %s' % e
+            return False
         except (URLError, IOError) as e:
             self._log(DEBUG2, 'urlopen() exception: %s', sys.exc_info())
             self._msg = str(e)
@@ -520,10 +547,11 @@ class PanWFapi:
         if not self.__set_response(response):
             raise PanWFapiError(self._msg)
 
-    def testfile(self):
+    def testfile(self, file_type=None):
         self.__clear_response()
 
-        request_uri = '/publicapi/test/pe'
+        request_uri = '/publicapi/test/%s' % (
+            'pe' if file_type is None else file_type)
 
         query = {}
 
@@ -850,6 +878,7 @@ class _FormDataPart:
 
         return bio.getvalue()
 
+
 if __name__ == '__main__':
     # python -m pan.wfapi [tag] [sha256]
     import pan.wfapi
